@@ -1,8 +1,8 @@
 #ifndef S7_H
 #define S7_H
 
-#define S7_VERSION "6.0"
-#define S7_DATE "23-Mar-18"
+#define S7_VERSION "7.0"
+#define S7_DATE "25-Jun-18"
 
 #include <stdint.h>           /* for int64_t */
 
@@ -227,7 +227,7 @@ s7_pointer s7_make_string(s7_scheme *sc, const char *str);                   /*
 s7_pointer s7_make_string_with_length(s7_scheme *sc, const char *str, s7_int len);  /* same as s7_make_string, but provides strlen */
 s7_pointer s7_make_string_wrapper(s7_scheme *sc, const char *str);
 s7_pointer s7_make_permanent_string(const char *str);                        /* make a string that will never be GC'd */
-s7_int s7_string_length(s7_pointer str);                                   /* (string-length str) */
+s7_int s7_string_length(s7_pointer str);                                     /* (string-length str) */
 
 
 bool s7_is_character(s7_pointer p);                                          /* (character? p) */
@@ -363,7 +363,8 @@ s7_pointer s7_make_symbol(s7_scheme *sc, const char *name);                 /* (
 s7_pointer s7_gensym(s7_scheme *sc, const char *prefix);                    /* (gensym prefix) */
 
 bool s7_is_keyword(s7_pointer obj);                                         /* (keyword? obj) */
-s7_pointer s7_make_keyword(s7_scheme *sc, const char *key);                 /* (make-keyword key) */
+s7_pointer s7_make_keyword(s7_scheme *sc, const char *key);                 /* (string->keyword key) */
+s7_pointer s7_keyword_to_symbol(s7_scheme *sc, s7_pointer key);             /* (keyword->symbol key) */
 
 s7_pointer s7_symbol_setter(s7_scheme *sc, s7_pointer sym);
 s7_pointer s7_symbol_set_setter(s7_scheme *sc, s7_pointer symbol, s7_pointer func);
@@ -417,16 +418,16 @@ bool s7_for_each_symbol(s7_scheme *sc, bool (*symbol_func)(const char *symbol_na
 
 s7_pointer s7_dynamic_wind(s7_scheme *sc, s7_pointer init, s7_pointer body, s7_pointer finish);
 
+bool s7_is_immutable(s7_pointer p);
+s7_pointer s7_immutable(s7_pointer p);
+
 void s7_define(s7_scheme *sc, s7_pointer env, s7_pointer symbol, s7_pointer value);
 bool s7_is_defined(s7_scheme *sc, const char *name);
 s7_pointer s7_define_variable(s7_scheme *sc, const char *name, s7_pointer value);
 s7_pointer s7_define_variable_with_documentation(s7_scheme *sc, const char *name, s7_pointer value, const char *help);
 s7_pointer s7_define_constant(s7_scheme *sc, const char *name, s7_pointer value);
 s7_pointer s7_define_constant_with_documentation(s7_scheme *sc, const char *name, s7_pointer value, const char *help);
-bool s7_is_immutable(s7_pointer p);
-s7_pointer s7_immutable(s7_pointer p);
-
-  /* These three functions add a symbol and its binding to either the top-level environment
+  /* These functions add a symbol and its binding to either the top-level environment
    *    or the 'env' passed as the second argument to s7_define.
    *
    *    s7_define_variable(sc, "*features*", sc->NIL);
@@ -583,54 +584,63 @@ s7_pointer s7_type_of(s7_pointer arg);                         /* (type-of arg)
 /* -------------------------------------------------------------------------------- */
 /* c types/objects */
 
+void s7_mark(s7_pointer p);
+
 bool s7_is_c_object(s7_pointer p);
 s7_int s7_c_object_type(s7_pointer obj);
 void *s7_c_object_value(s7_pointer obj);
 void *s7_c_object_value_checked(s7_pointer obj, s7_int type);
 s7_pointer s7_make_c_object(s7_scheme *sc, s7_int type, void *value);
 s7_pointer s7_make_c_object_with_let(s7_scheme *sc, s7_int type, void *value, s7_pointer let);
-void s7_mark_c_object(s7_pointer p);
 s7_pointer s7_c_object_let(s7_pointer obj);
 s7_pointer s7_c_object_set_let(s7_pointer obj, s7_pointer e);
 
 s7_int s7_make_c_type(s7_scheme *sc, const char *name);
-void s7_c_type_set_print         (s7_scheme *sc, s7_int tag, char *(*print)(s7_scheme *sc, void *value));
-void s7_c_type_set_print_readably(s7_scheme *sc, s7_int tag, char *(*printer)(s7_scheme *sc, void *val));
-void s7_c_type_set_free          (s7_scheme *sc, s7_int tag, void (*gc_free)(void *value));
-void s7_c_type_set_equal         (s7_scheme *sc, s7_int tag, bool (*equal)(void *val1, void *val2));
-void s7_c_type_set_mark          (s7_scheme *sc, s7_int tag, void (*gc_mark)(void *val));
-void s7_c_type_set_apply         (s7_scheme *sc, s7_int tag, s7_pointer (*ref)    (s7_scheme *sc, s7_pointer obj, s7_pointer args));
-void s7_c_type_set_apply_direct  (s7_scheme *sc, s7_int tag, s7_pointer (*dref)   (s7_scheme *sc, s7_pointer obj, s7_int index));
-void s7_c_type_set_set           (s7_scheme *sc, s7_int tag, s7_pointer (*set)    (s7_scheme *sc, s7_pointer obj, s7_pointer args));
-void s7_c_type_set_set_direct    (s7_scheme *sc, s7_int tag, s7_pointer (*dset)   (s7_scheme *sc, s7_pointer obj, s7_int index, s7_pointer val));
-void s7_c_type_set_length        (s7_scheme *sc, s7_int tag, s7_pointer (*length) (s7_scheme *sc, s7_pointer obj));
-void s7_c_type_set_copy          (s7_scheme *sc, s7_int tag, s7_pointer (*copy)   (s7_scheme *sc, s7_pointer args));
-void s7_c_type_set_fill          (s7_scheme *sc, s7_int tag, s7_pointer (*fill)   (s7_scheme *sc, s7_pointer args));
-void s7_c_type_set_reverse       (s7_scheme *sc, s7_int tag, s7_pointer (*reverse)(s7_scheme *sc, s7_pointer args));
+void s7_c_type_set_free     (s7_scheme *sc, s7_int tag, void (*gc_free)(void *value));
+void s7_c_type_set_equal    (s7_scheme *sc, s7_int tag, bool (*equal)(void *value1, void *value2));
+void s7_c_type_set_mark     (s7_scheme *sc, s7_int tag, void (*mark)(void *value));
+void s7_c_type_set_ref      (s7_scheme *sc, s7_int tag, s7_pointer (*ref)      (s7_scheme *sc, s7_pointer args));
+void s7_c_type_set_set      (s7_scheme *sc, s7_int tag, s7_pointer (*set)      (s7_scheme *sc, s7_pointer args));
+void s7_c_type_set_length   (s7_scheme *sc, s7_int tag, s7_pointer (*length)   (s7_scheme *sc, s7_pointer args));
+void s7_c_type_set_copy     (s7_scheme *sc, s7_int tag, s7_pointer (*copy)     (s7_scheme *sc, s7_pointer args));
+void s7_c_type_set_fill     (s7_scheme *sc, s7_int tag, s7_pointer (*fill)     (s7_scheme *sc, s7_pointer args));
+void s7_c_type_set_reverse  (s7_scheme *sc, s7_int tag, s7_pointer (*reverse)  (s7_scheme *sc, s7_pointer args));
+void s7_c_type_set_to_list  (s7_scheme *sc, s7_int tag, s7_pointer (*to_list)  (s7_scheme *sc, s7_pointer args));
+void s7_c_type_set_to_string(s7_scheme *sc, s7_int tag, s7_pointer (*to_string)(s7_scheme *sc, s7_pointer args));
+
+/* if a c-object might participate in a cyclic structure, and you want to check its equality to another such object
+ *   or get a readable string representing that object, you need to implement the "to_list" and "set" cases above,
+ *   and make the type name a function that can recreate the object.  See the <cycle> object in s7test.scm.
+ *   For the copy function, either the first or second argument can be a c-object of the given type.
+ */
+
+#if (!DISABLE_DEPRECATED)
+  void s7_c_type_set_print       (s7_scheme *sc, s7_int tag, char *(*print)(s7_scheme *sc, void *value));
+#endif
 
   /* These functions create a new Scheme object type.  There is a simple example in s7.html.
    *
    * s7_make_c_type creates a new C-based type for Scheme:
-   *   print:   the function called whenever s7 is asked to display a value with this type
    *   free:    the function called when an object of this type is about to be garbage collected
+   *   mark:    called during the GC mark pass -- you should call s7_mark
+   *            on any embedded s7_pointer associated with the object to protect if from the GC.
    *   equal:   compare two objects of this type; (equal? obj1 obj2)
-   *   gc_mark: called during the GC mark pass -- you should call s7_mark_object
-   *            on any embedded s7_pointer associated with the object.
-   *   apply:   a function that is called whenever an object of this type
+   *   ref:     a function that is called whenever an object of this type
    *            occurs in the function position (at the car of a list; the rest of the list
-   *            is passed to the apply function as the arguments).
+   *            is passed to the ref function as the arguments: (obj ...))
    *   set:     a function that is called whenever an object of this type occurs as
-   *            the target of a generalized set!
+   *            the target of a generalized set! (set! (obj ...) val)
    *   length:  the function called when the object is asked what its length is.
    *   copy:    the function called when a copy of the object is needed.
    *   fill:    the function called to fill the object with some value.
    *   reverse: similarly...
+   *   to_string: object->string for an object of this type
    *
    * s7_is_c_object returns true if 'p' holds a value of a type created by s7_new_type.
    * s7_c_object_type returns the c_object's type
    * s7_c_object_value returns the value bound to that c_object (the void *value of s7_make_c_object)
    * s7_make_c_object creates a new Scheme entity of the given type with the given (uninterpreted) value
-   * s7_mark_c_object marks any Scheme c_object as in-use (use this in the gc_mark function to mark
+   * s7_mark marks any Scheme c_object as in-use (use this in the mark function to mark
    *    any embedded s7_pointer variables).
    */
 
@@ -736,6 +746,29 @@ typedef s7_double (*s7_d_pi_t)(s7_pointer v, s7_int i);
 void s7_set_d_pi_function(s7_pointer f, s7_d_pi_t df);
 s7_d_pi_t s7_d_pi_function(s7_pointer f);
 
+/* Here is an example of using these functions; more extensive examples are in clm2xen.c in sndlib, and in s7.c.
+ * (This example comes from a HackerNews discussion):
+ * plus.c:
+ * --------
+ * #include "s7.h"
+ * 
+ * s7_pointer g_plusone(s7_scheme *sc, s7_pointer args) {return(s7_make_integer(sc, s7_integer(s7_car(args)) + 1));}
+ * s7_int plusone(s7_int x) {return(x + 1);}
+ * 
+ * void plusone_init(s7_scheme *sc)
+ * {
+ *   s7_define_safe_function(sc, "plusone", g_plusone, 1, 0, false, "");
+ *   s7_set_i_i_function(s7_name_to_value(sc, "plusone"), plusone);
+ * }
+ * --------
+ * gcc -c plus.c -fPIC -O2 -lm
+ * gcc plus.o -shared -o plus.so -ldl -lm -Wl,-export-dynamic
+ * repl
+ * <1> (load "plus.so" (inlet 'init_func 'plusone_init))
+ * --------
+ */
+
+
 /* -------------------------------------------------------------------------------- */
 
 
@@ -807,9 +840,6 @@ s7_pointer s7_apply_n_9(s7_scheme *sc, s7_pointer args,
 #define s7_make_ulong_long(sc, n) s7_make_c_pointer(sc, (void *)n)
 
 #else
-int64_t s7_heap_size(s7_scheme *sc);
-int64_t s7_gc_freed(s7_scheme *sc);
-
 bool s7_is_ulong(s7_pointer arg);
 unsigned long s7_ulong(s7_pointer p);
 s7_pointer s7_make_ulong(s7_scheme *sc, unsigned long n);
@@ -837,21 +867,18 @@ s7_int s7_new_type(const char *name,
 		   char *(*print)(s7_scheme *sc, void *value), 
 		   void (*free)(void *value), 
 		   bool (*equal)(void *val1, void *val2),
-		   void (*gc_mark)(void *val),
-		   s7_pointer (*apply)(s7_scheme *sc, s7_pointer obj, s7_pointer args),
+		   void (*mark)(void *val),
+		   s7_pointer (*apply)(s7_scheme *sc, s7_pointer obj, s7_pointer args), /* these two args are ignored */
 		   s7_pointer (*set)(s7_scheme *sc, s7_pointer obj, s7_pointer args));
 
-void s7_object_type_set_direct(s7_int tag, 
-			       s7_pointer (*dref)(s7_scheme *sc, s7_pointer obj, s7_int index), 
-			       s7_pointer (*dset)(s7_scheme *sc, s7_pointer obj, s7_int index, s7_pointer val));
-
 #define s7_is_object                 s7_is_c_object
 #define s7_object_type               s7_c_object_type
 #define s7_object_value              s7_c_object_value
 #define s7_object_value_checked      s7_c_object_value_checked
 #define s7_make_object               s7_make_c_object
 #define s7_make_object_with_let      s7_make_c_object_with_let
-#define s7_mark_object               s7_mark_c_object
+#define s7_mark_object               s7_mark
+#define s7_mark_c_object             s7_mark
 #define s7_object_let                s7_c_object_let
 #define s7_object_set_let            s7_c_object_set_let
 #define s7_set_object_print_readably s7_c_type_set_print_readably
@@ -873,6 +900,10 @@ void s7_define_function_with_setter(s7_scheme *sc, const char *name, s7_function
  * 
  *        s7 changes
  *
+ * 20-May:    s7_keyword_to_symbol.
+ * 6-May:     s7_mark_c_object -> s7_mark.
+ * 26-Apr:    s7_c_type_set_to_list|string, s7_c_type_set_apply -> s7_c_type_set_ref, removed s7_c_type_set_set|apply_direct
+ *            c_type length|set|ref are now s7_functions (args, not obj, etc).
  * 23-Mar:    s7_peek_char and s7_read_char now return s7_pointer, s7_write_char takes s7_pointer, not int32_t c
  *            s7_gc_protect and friends now return/take s7_int location, not uint32_t.
  *            removed s7_new_type_x.
