@@ -2015,23 +2015,33 @@ static char *xen_mark_to_string(xen_mark *v)
   return(buf);
 }
 
-Xen_wrap_print(xen_mark, print_xen_mark, xen_mark_to_string)
-
 
 #if HAVE_FORTH || HAVE_RUBY
+Xen_wrap_print(xen_mark, print_xen_mark, xen_mark_to_string)
+
 static Xen g_xen_mark_to_string(Xen obj)
 {
   char *vstr;
   Xen result;
   #define S_xen_mark_to_string "mark->string"
-
   Xen_check_type(xen_is_mark(obj), obj, 1, S_xen_mark_to_string, "a mark");
-
   vstr = xen_mark_to_string(Xen_to_xen_mark(obj));
   result = C_string_to_Xen_string(vstr);
   free(vstr);
   return(result);
 }
+#else
+#if HAVE_SCHEME
+static s7_pointer g_xen_mark_to_string(s7_scheme *sc, s7_pointer args)
+{
+  char *vstr;
+  s7_pointer result;
+  vstr = xen_mark_to_string(Xen_to_xen_mark(s7_car(args)));
+  result = C_string_to_Xen_string(vstr);
+  free(vstr);
+  return(result);
+}
+#endif
 #endif
 
 
@@ -2119,10 +2129,10 @@ static void init_xen_mark(void)
   g_mark_methods = s7_openlet(s7, s7_inlet(s7, s7_list(s7, 2, s7_make_symbol(s7, "object->let"), mark_to_let_func)));
   s7_gc_protect(s7, g_mark_methods);
   xen_mark_tag = s7_make_c_type(s7, "<mark>");
-  s7_c_type_set_print(s7, xen_mark_tag, print_xen_mark);
   s7_c_type_set_free(s7, xen_mark_tag, free_xen_mark);
   s7_c_type_set_equal(s7, xen_mark_tag, s7_xen_mark_equalp);
   s7_c_type_set_copy(s7, xen_mark_tag, s7_xen_mark_copy);
+  s7_c_type_set_to_string(s7, xen_mark_tag, g_xen_mark_to_string);
 #else
 #if HAVE_RUBY
   xen_mark_tag = Xen_make_object_type("XenMark", sizeof(xen_mark));
@@ -2734,20 +2744,20 @@ static mark *save_mark(chan_info *cp, mark *m, void *info)
 
 #if HAVE_SCHEME
   if (m->name)
-    fprintf(sv->fd, "(add-mark! %" PRId64 " sfile %d \"%s\" %s)\n", m->samp, cp->chan, m->name, mapped_sync);
-  else fprintf(sv->fd, "(add-mark! %" PRId64 " sfile %d #f %s)\n", m->samp, cp->chan, mapped_sync);
+    fprintf(sv->fd, "(add-mark! %" print_mus_long " sfile %d \"%s\" %s)\n", m->samp, cp->chan, m->name, mapped_sync);
+  else fprintf(sv->fd, "(add-mark! %" print_mus_long " sfile %d #f %s)\n", m->samp, cp->chan, mapped_sync);
 #endif
 
 #if HAVE_RUBY
   if (m->name)
-    fprintf(sv->fd, "add_mark!(%" PRId64 ", sfile, %d, \"%s\", %s)\n", m->samp, cp->chan, m->name, mapped_sync);
-  else fprintf(sv->fd, "add_mark!(%" PRId64 ", sfile, %d, false, %s)\n", m->samp, cp->chan, mapped_sync);
+    fprintf(sv->fd, "add_mark!(%" print_mus_long ", sfile, %d, \"%s\", %s)\n", m->samp, cp->chan, m->name, mapped_sync);
+  else fprintf(sv->fd, "add_mark!(%" print_mus_long ", sfile, %d, false, %s)\n", m->samp, cp->chan, mapped_sync);
 #endif
 
 #if HAVE_FORTH
   if (m->name)
-    fprintf(sv->fd, "%" PRId64 " sfile %d \"%s\" %s add-mark! drop\n", m->samp, cp->chan, m->name, mapped_sync);
-  else fprintf(sv->fd, "%" PRId64 " sfile %d #f %s add-mark! drop\n", m->samp, cp->chan, mapped_sync);
+    fprintf(sv->fd, "%" print_mus_long " sfile %d \"%s\" %s add-mark! drop\n", m->samp, cp->chan, m->name, mapped_sync);
+  else fprintf(sv->fd, "%" print_mus_long " sfile %d #f %s add-mark! drop\n", m->samp, cp->chan, mapped_sync);
 #endif
 
   free(mapped_sync);
