@@ -514,6 +514,7 @@ region_state *region_report(void)
 {
   region_state *rs;
   int i, len;
+  size_t size;
   rs = (region_state *)calloc(1, sizeof(region_state));
   len = regions_size;
   for (i = 0; i < regions_size; i++) 
@@ -524,7 +525,8 @@ region_state *region_report(void)
       }
   rs->len = len;
   if (len == 0) return(rs);
-  rs->name = (char **)calloc(len, sizeof(char *));
+  size = len * sizeof(char *);
+  rs->name = (char **)calloc(size, 1);
   for (i = 0; i < len; i++)
     {
       region *r;
@@ -645,13 +647,13 @@ static int paste_region_1(int n, chan_info *cp, bool add, mus_long_t beg, io_err
       else 
 	{
 #if HAVE_FORTH
-	  origin = mus_format("%d %s %" PRId64 " %s drop", n, S_integer_to_region, beg, S_mix_region);
+	  origin = mus_format("%d %s %" print_mus_long " %s drop", n, S_integer_to_region, beg, S_mix_region);
 #endif
 #if HAVE_RUBY
-	  origin = mus_format("%s(%s(%d), %" PRId64, to_proc_name(S_mix_region), to_proc_name(S_integer_to_region), n, beg);
+	  origin = mus_format("%s(%s(%d), %" print_mus_long, to_proc_name(S_mix_region), to_proc_name(S_integer_to_region), n, beg);
 #endif
 #if HAVE_SCHEME || (!HAVE_EXTENSION_LANGUAGE)
-	  origin = mus_format("%s (%s %d) %" PRId64, S_mix_region, S_integer_to_region, n, beg);
+	  origin = mus_format("%s (%s %d) %" print_mus_long, S_mix_region, S_integer_to_region, n, beg);
 #endif
 	  if (si->chans > 1)
 	    remember_temp(newname, si->chans);
@@ -683,13 +685,13 @@ static int paste_region_1(int n, chan_info *cp, bool add, mus_long_t beg, io_err
 	}
 
 #if HAVE_FORTH
-	  origin = mus_format("%d %s %" PRId64 " %s drop", n, S_integer_to_region, beg, S_insert_region);
+	  origin = mus_format("%d %s %" print_mus_long " %s drop", n, S_integer_to_region, beg, S_insert_region);
 #endif
 #if HAVE_RUBY
-	  origin = mus_format("%s(%s(%d), %" PRId64, to_proc_name(S_insert_region), to_proc_name(S_integer_to_region), n, beg);
+	  origin = mus_format("%s(%s(%d), %" print_mus_long, to_proc_name(S_insert_region), to_proc_name(S_integer_to_region), n, beg);
 #endif
 #if HAVE_SCHEME || (!HAVE_EXTENSION_LANGUAGE)
-	  origin = mus_format("%s (%s %d) %" PRId64, S_insert_region, S_integer_to_region, n, beg);
+	  origin = mus_format("%s (%s %d) %" print_mus_long, S_insert_region, S_integer_to_region, n, beg);
 #endif
 
       for (i = 0; ((i < r->chans) && (i < si->chans)); i++)
@@ -1123,25 +1125,25 @@ void save_regions(FILE *fd)
 	  else
 	    {
 #if HAVE_RUBY
-	      fprintf(fd, "%s(%d, %d, %" PRId64 ", %d, %.4f, \"%s\", \"%s\", \"%s\", ",
+	      fprintf(fd, "%s(%d, %d, %" print_mus_long ", %d, %.4f, \"%s\", \"%s\", \"%s\", ",
 		      "restore_region", i, r->chans, r->framples, r->srate, r->maxamp, r->name, r->start, r->end);
-	      fprintf(fd, " \"%s\", [%d, %" PRId64 "])\n",
+	      fprintf(fd, " \"%s\", [%d, %" print_mus_long "])\n",
 		      newname,
 		      (int)mus_sound_write_date(newname),
 		      mus_sound_length(newname));
 #endif
 #if HAVE_SCHEME
-	      fprintf(fd, "(%s %d %d %" PRId64 " %d %.4f \"%s\" \"%s\" \"%s\"",
+	      fprintf(fd, "(%s %d %d %" print_mus_long " %d %.4f \"%s\" \"%s\" \"%s\"",
 		      S_restore_region, i, r->chans, r->framples, r->srate, r->maxamp, r->name, r->start, r->end);
-	      fprintf(fd, " \"%s\" (list %d %" PRId64 "))\n",
+	      fprintf(fd, " \"%s\" (list %d %" print_mus_long "))\n",
 		      newname,
 		      (int)mus_sound_write_date(newname),
 		      mus_sound_length(newname));
 #endif
 #if HAVE_FORTH
-	  fprintf(fd, "%d %d %" PRId64 " %d %.4f \"%s\" \"%s\" \"%s\"",
+	  fprintf(fd, "%d %d %" print_mus_long " %d %.4f \"%s\" \"%s\" \"%s\"",
 	          i, r->chans, r->framples, r->srate, r->maxamp, r->name, r->start, r->end);
- 	  fprintf(fd, " \"%s\" '( %d %" PRId64 " ) %s drop\n",
+ 	  fprintf(fd, " \"%s\" '( %d %" print_mus_long " ) %s drop\n",
  		  newname,
  		  (int)mus_sound_write_date(newname),
  		  mus_sound_length(newname),
@@ -1327,7 +1329,7 @@ io_error_t save_region(int rg, const char *name, mus_sample_t samp_type, mus_hea
 	      for (i = 0; i < chans; i++) bufs[i] = (mus_float_t *)calloc(FILE_BUFFER_SIZE, sizeof(mus_float_t));
 
 	      if (((framples * chans * mus_sound_datum_size(r->filename)) >> 10) > disk_kspace(name))
-		snd_warning("not enough space to save region? -- need %" PRId64 " bytes",
+		snd_warning("not enough space to save region? -- need %" print_mus_long " bytes",
 			    framples * chans * mus_sound_datum_size(r->filename));
 
 	      for (ioff = 0; ioff < framples; ioff += FILE_BUFFER_SIZE)
@@ -1417,23 +1419,33 @@ static char *xen_region_to_string(xen_region *v)
   return(buf);
 }
 
-Xen_wrap_print(xen_region, print_xen_region, xen_region_to_string)
-
 
 #if HAVE_FORTH || HAVE_RUBY
+Xen_wrap_print(xen_region, print_xen_region, xen_region_to_string)
+
 static Xen g_xen_region_to_string(Xen obj)
 {
   char *vstr;
   Xen result;
   #define S_xen_region_to_string "region->string"
-
   Xen_check_type(xen_is_region(obj), obj, 1, S_xen_region_to_string, "a region");
-
   vstr = xen_region_to_string(Xen_to_xen_region(obj));
   result = C_string_to_Xen_string(vstr);
   free(vstr);
   return(result);
 }
+#else
+#if HAVE_SCHEME
+static s7_pointer g_xen_region_to_string(s7_scheme *sc, s7_pointer args)
+{
+  char *vstr;
+  Xen result;
+  vstr = xen_region_to_string(Xen_to_xen_region(s7_car(args)));
+  result = C_string_to_Xen_string(vstr);
+  free(vstr);
+  return(result);
+}
+#endif
 #endif
 
 
@@ -1479,9 +1491,9 @@ static bool s7_xen_region_equalp(void *obj1, void *obj2)
 	 (((xen_region *)obj1)->n == ((xen_region *)obj2)->n));
 }
 
-static Xen s7_xen_region_length(s7_scheme *sc, Xen obj)
+static Xen s7_xen_region_length(s7_scheme *sc, Xen args)
 {
-  return(g_region_framples(obj, Xen_integer_zero));
+  return(g_region_framples(s7_car(args), Xen_integer_zero));
 }
 #endif
 
@@ -1490,10 +1502,10 @@ static void init_xen_region(void)
 {
 #if HAVE_SCHEME
   xen_region_tag = s7_make_c_type(s7, "<region>");
-  s7_c_type_set_print(s7, xen_region_tag, print_xen_region);
   s7_c_type_set_free(s7, xen_region_tag, free_xen_region);
   s7_c_type_set_equal(s7, xen_region_tag, s7_xen_region_equalp);
   s7_c_type_set_length(s7, xen_region_tag, s7_xen_region_length);
+  s7_c_type_set_to_string(s7, xen_region_tag, g_xen_region_to_string);
 #else
 #if HAVE_RUBY
   xen_region_tag = Xen_make_object_type("XenRegion", sizeof(xen_region));
