@@ -55,11 +55,11 @@
 
 #include "snd-strings.h"
 
-#define SND_DATE "24-Apr-18"
+#define SND_DATE "1-July-18"
 #ifndef SND_VERSION
-#define SND_VERSION "18.3"
+#define SND_VERSION "18.5"
 #endif
 #define SND_MAJOR_VERSION "18"
-#define SND_MINOR_VERSION "3"
+#define SND_MINOR_VERSION "5"
 
 #endif
