@@ -11,6 +11,8 @@
 
 #include "s7.h"
 
+#define print_s7_int PRId64
+
 #define TO_STR(x) s7_object_to_c_string(sc, x)
 #define TO_S7_INT(x) s7_make_integer(sc, x)
 
@@ -72,17 +74,19 @@ typedef struct {
   s7_pointer data;
 } dax;
 
-static char *print_dax(s7_scheme *sc, void *val)
+static s7_pointer dax_to_string(s7_scheme *sc, s7_pointer args)
 {
   char *data_str, *str;
+  s7_pointer result;
   int data_str_len;
-  dax *o = (dax *)val;
+  dax *o = (dax *)s7_c_object_value(s7_car(args));
   data_str = s7_object_to_c_string(sc, o->data);
   data_str_len = strlen(data_str);
   str = (char *)calloc(data_str_len + 32, sizeof(char));
   snprintf(str, data_str_len + 32, "#<dax %.3f %s>", o->x, data_str);
-  free(data_str);
-  return(str);
+  result = s7_make_string(sc, str);
+  free(str);
+  return(result);
 }
 
 static void free_dax(void *val)
@@ -98,7 +102,7 @@ static bool equal_dax(void *val1, void *val2)
 static void mark_dax(void *val)
 {
   dax *o = (dax *)val;
-  if (o) s7_mark_c_object(o->data);
+  if (o) s7_mark(o->data);
 }
 
 static s7_int dax_type_tag = 0;
@@ -291,9 +295,9 @@ static s7_pointer g_to_block(s7_scheme *sc, s7_pointer args)
   return(b);
 }
 
-static char *g_block_display(s7_scheme *sc, void *value)
+static s7_pointer g_block_to_string(s7_scheme *sc, s7_pointer args)
 {
-  return(strdup("#<block>"));
+  return(s7_make_string(sc, "<block>"));
 }
 
 static void g_block_free(void *value)
@@ -313,32 +317,34 @@ static void g_block_mark(void *val)
   /* nothing to mark */
 }
 
-static s7_pointer g_block_ref(s7_scheme *sc, s7_pointer obj, s7_pointer args)
+static s7_pointer g_block_ref(s7_scheme *sc, s7_pointer args)
 {
-  g_block *g = (g_block *)s7_c_object_value(obj);
+  g_block *g;
   size_t index;
-  index = (size_t)s7_integer(s7_car(args));
+  g = (g_block *)s7_c_object_value(s7_car(args));
+  index = (size_t)s7_integer(s7_cadr(args));
   if (index < g->size)
     return(s7_make_real(sc, g->data[index]));
-  return(s7_out_of_range_error(sc, "block-ref", 2, s7_car(args), "should be less than block length"));
+  return(s7_out_of_range_error(sc, "block-ref", 2, s7_cadr(args), "should be less than block length"));
 }
 
-static s7_pointer g_block_set(s7_scheme *sc, s7_pointer obj, s7_pointer args)
+static s7_pointer g_block_set(s7_scheme *sc, s7_pointer args)
 {
-  g_block *g = (g_block *)s7_c_object_value(obj);
+  g_block *g;
   s7_int index;
-  index = s7_integer(s7_car(args));
+  g = (g_block *)s7_c_object_value(s7_car(args));
+  index = s7_integer(s7_cadr(args));
   if ((index >= 0) && (index < g->size))
     {
-      g->data[index] = s7_number_to_real(sc, s7_cadr(args));
-      return(s7_cadr(args));
+      g->data[index] = s7_number_to_real(sc, s7_caddr(args));
+      return(s7_caddr(args));
     }
-  return(s7_out_of_range_error(sc, "block-set", 2, s7_car(args), "should be less than block length"));
+  return(s7_out_of_range_error(sc, "block-set", 2, s7_cadr(args), "should be less than block length"));
 }
 
-static s7_pointer g_block_length(s7_scheme *sc, s7_pointer obj)
+static s7_pointer g_block_length(s7_scheme *sc, s7_pointer args)
 {
-  g_block *g = (g_block *)s7_c_object_value(obj);
+  g_block *g = (g_block *)s7_c_object_value(s7_car(args));
   return(s7_make_integer(sc, g->size));
 }
 
@@ -354,12 +360,13 @@ static s7_pointer g_block_copy(s7_scheme *sc, s7_pointer args)
   return(new_g);
 }
 
-static s7_pointer g_block_reverse(s7_scheme *sc, s7_pointer obj)
+static s7_pointer g_block_reverse(s7_scheme *sc, s7_pointer args)
 {
   size_t i, j;
-  g_block *g = (g_block *)s7_c_object_value(obj);
-  g_block *g1;
-  s7_pointer new_g;
+  s7_pointer obj, new_g;
+  g_block *g, *g1;
+  obj = s7_car(args);
+  g = (g_block *)s7_c_object_value(obj);
   new_g = g_make_block(sc, s7_cons(sc, s7_make_integer(sc, g->size), s7_nil(sc)));
   g1 = (g_block *)s7_c_object_value(new_g);
   for (i = 0, j = g->size - 1; i < g->size; i++, j--)
@@ -538,7 +545,7 @@ int main(int argc, char **argv)
 
   i = (*((int *)s7_c_pointer(p)));
   if (i != 32)
-    fprintf(stderr, "%d: 32 -> %" PRId64 " via raw c pointer?\n", __LINE__, i);
+    fprintf(stderr, "%d: 32 -> %" print_s7_int " via raw c pointer?\n", __LINE__, i);
 
   s7_provide(sc, "ffitest");
   if (!s7_is_provided(sc, "ffitest"))
@@ -547,7 +554,7 @@ int main(int argc, char **argv)
   p = s7_cons(sc, s7_f(sc), s7_t(sc));
   gc_loc = s7_gc_protect(sc, p);
   if (p != s7_gc_protected_at(sc, gc_loc))
-    {fprintf(stderr, "%d: %s is not gc protected at %" PRId64 ": %s?\n", __LINE__, s1 = TO_STR(p), gc_loc, s2 = TO_STR(s7_gc_protected_at(sc, gc_loc))); free(s1); free(s2);}
+    {fprintf(stderr, "%d: %s is not gc protected at %" print_s7_int ": %s?\n", __LINE__, s1 = TO_STR(p), gc_loc, s2 = TO_STR(s7_gc_protected_at(sc, gc_loc))); free(s1); free(s2);}
   
   if (s7_car(p) != s7_f(sc))
     {fprintf(stderr, "%d: (car %s) is not #f?\n", __LINE__, s1 = TO_STR(p)); free(s1);}
@@ -745,9 +752,9 @@ int main(int argc, char **argv)
   {
     int64_t size;
     size = s7_heap_size(sc);
-    if (size <= 0) fprintf(stderr, "heap_size: %" PRId64 "?\n", size);
+    if (size <= 0) fprintf(stderr, "heap_size: %" print_s7_int "?\n", size);
     size = s7_gc_freed(sc);
-    if (size < 0) fprintf(stderr, "gc_freed: %" PRId64 "?\n", size);
+    if (size < 0) fprintf(stderr, "gc_freed: %" print_s7_int "?\n", size);
     s7_gc_stats(sc, false);
   }
 #endif
@@ -1143,10 +1150,10 @@ int main(int argc, char **argv)
 
 
   dax_type_tag = s7_make_c_type(sc, "dax");
-  s7_c_type_set_print(sc, dax_type_tag, print_dax);
   s7_c_type_set_free(sc, dax_type_tag, free_dax);
   s7_c_type_set_equal(sc, dax_type_tag, equal_dax);
   s7_c_type_set_mark(sc, dax_type_tag, mark_dax);
+  s7_c_type_set_to_string(sc, dax_type_tag, dax_to_string);
 
   s7_define_function(sc, "make-dax", make_dax, 2, 0, false, "(make-dax x data) makes a new dax");
   s7_define_function(sc, "dax?", is_dax, 1, 0, false, "(dax? anything) returns #t if its argument is a dax object");
@@ -1210,7 +1217,7 @@ int main(int argc, char **argv)
 						    s7_make_integer(sc, 5),
 						    s7_make_integer(sc, 6))));
     if (val != 21)
-      fprintf(stderr, "plus1: %" PRId64 "\n", val);
+      fprintf(stderr, "plus1: %" print_s7_int "\n", val);
   }
   
   p = s7_apply_function(sc, s7_name_to_value(sc, "plus"), s7_cons(sc, s7_make_keyword(sc, "blue"), s7_cons(sc, TO_S7_INT(2), s7_nil(sc))));
@@ -1296,11 +1303,11 @@ int main(int argc, char **argv)
     dims = s7_vector_dimensions(p1);
     offs = s7_vector_offsets(p1);
     els = s7_vector_elements(p1);
-    if (dims[0] != 2) fprintf(stderr, "%d: dims[0]: %" PRId64 "?\n", __LINE__, dims[0]);
-    if (dims[1] != 3) fprintf(stderr, "%d: dims[1]: %" PRId64 "?\n", __LINE__, dims[1]);
-    if (dims[2] != 4) fprintf(stderr, "%d: dims[2]: %" PRId64 "?\n", __LINE__, dims[2]);
-    if (offs[0] != 12) fprintf(stderr, "%d: offs[0]: %" PRId64 "?\n", __LINE__, offs[0]);
-    if (offs[1] != 4) fprintf(stderr, "%d: offs[1]: %" PRId64 "?\n", __LINE__, offs[1]);
+    if (dims[0] != 2) fprintf(stderr, "%d: dims[0]: %" print_s7_int "?\n", __LINE__, dims[0]);
+    if (dims[1] != 3) fprintf(stderr, "%d: dims[1]: %" print_s7_int "?\n", __LINE__, dims[1]);
+    if (dims[2] != 4) fprintf(stderr, "%d: dims[2]: %" print_s7_int "?\n", __LINE__, dims[2]);
+    if (offs[0] != 12) fprintf(stderr, "%d: offs[0]: %" print_s7_int "?\n", __LINE__, offs[0]);
+    if (offs[1] != 4) fprintf(stderr, "%d: offs[1]: %" print_s7_int "?\n", __LINE__, offs[1]);
     if (s7_integer(p = els[12 + 4 + 1]) != 32)
       {fprintf(stderr, "%d: %s is not 32?\n", __LINE__, s1 = TO_STR(p)); free(s1);}
   }
@@ -1682,17 +1689,17 @@ int main(int argc, char **argv)
       fprintf(stderr, "%d: %s should be #<eof> and iter should be done\n", __LINE__, TO_STR(x));
   }
 
-  g_block_type = s7_make_c_type(sc, "#<block>");
-  s7_c_type_set_print(sc, g_block_type, g_block_display);
+  g_block_type = s7_make_c_type(sc, "<block>");
   s7_c_type_set_free(sc, g_block_type, g_block_free);
   s7_c_type_set_equal(sc, g_block_type, g_block_is_equal);
   s7_c_type_set_mark(sc, g_block_type, g_block_mark);
-  s7_c_type_set_apply(sc, g_block_type, g_block_ref);
+  s7_c_type_set_ref(sc, g_block_type, g_block_ref);
   s7_c_type_set_set(sc, g_block_type, g_block_set);
   s7_c_type_set_length(sc, g_block_type, g_block_length);
   s7_c_type_set_copy(sc, g_block_type, g_block_copy);
   s7_c_type_set_reverse(sc, g_block_type, g_block_reverse);
   s7_c_type_set_fill(sc, g_block_type, g_block_fill);
+  s7_c_type_set_to_string(sc, g_block_type, g_block_to_string);
 
   s7_define_function(sc, "make-block", g_make_block, 1, 0, false, g_make_block_help);
   s7_define_function(sc, "block", g_to_block, 0, 0, true, g_block_help);
@@ -1710,9 +1717,9 @@ int main(int argc, char **argv)
       {fprintf(stderr, "%d: g_block %s is not a c_object?\n", __LINE__, s1 = TO_STR(gp)); free(s1);}
     g = (g_block *)s7_c_object_value(gp);
     if (s7_c_object_type(gp) != g_block_type)
-      {fprintf(stderr, "%d: g_block types: %" PRId64 " %" PRId64 "\n", __LINE__, g_block_type, s7_c_object_type(gp));}
+      {fprintf(stderr, "%d: g_block types: %" print_s7_int " %" print_s7_int "\n", __LINE__, g_block_type, s7_c_object_type(gp));}
     if (s7_c_object_value_checked(gp, g_block_type) != g)
-      {fprintf(stderr, "%d: checked g_block types: %" PRId64 " %" PRId64 "\n", __LINE__, g_block_type, s7_c_object_type(gp));}
+      {fprintf(stderr, "%d: checked g_block types: %" print_s7_int " %" print_s7_int "\n", __LINE__, g_block_type, s7_c_object_type(gp));}
 
     s7_gc_unprotect_at(sc, gc_loc);
   }
