@@ -1,5 +1,6 @@
 (define file-names '(("make-index.scm" . "v-index")
 		     ("tmac.scm" . "v-mac")
+		     ("tpeak.scm" . "v-peak")
 		     ("teq.scm" . "v-eq")
 		     ("tfft.scm" . "v-fft")
 		     ("tref.scm" . "v-ref")
@@ -9,6 +10,7 @@
 		     ("tcopy.scm" . "v-cop")
 		     ("lt.scm" . "v-lt")
 		     ("tform.scm" . "v-form")
+		     ("tread.scm" . "v-read")
 		     ("tmap.scm" . "v-map")
 		     ;("lg.scm" . "v-lg")
 		     ("titer.scm" . "v-iter")
@@ -48,25 +50,26 @@
      ;;   comment out lines 1333 to 1341
      (let ((outfile (cdr (assoc (cadr caller+file) file-names))))
        (let ((next (next-file outfile)))
-	 (system (format #f "/home/bil/test/valgrind-3.12.0/callgrind/callgrind_annotate --auto=yes --threshold=100 ~A > ~A~D" (last-callg) outfile next))
+	 (system (format #f "callgrind_annotate --auto=yes --threshold=100 ~A > ~A~D" (last-callg) outfile next))
 	 ;; new callgrind blathers endlessly -- I made this change:
          ;;   (line 825) my $space = ' ' x ($CC_col_widths->[$i] - length($count));
          ;;              my $space = ' ' x max($CC_col_widths->[$i] - length($count), 0);
 	 (format *stderr* "~NC ~A~D -> ~A~D: ~NC~%" 8 #\space outfile (- next 1) outfile next 8 #\space)
-	 (system (format #f "./snd compare-calls.scm -e '(compare-calls \"~A~D\" \"~A~D\")'" outfile (- next 1) outfile next))))
-     (system (format #f "/home/bil/test/valgrind-3.12.0/vg-in-place ./~A ~A" (car caller+file) (cadr caller+file))))
+	 (system (format #f "./snd compare-calls.scm -e '(compare-calls \"~A~D\" \"~A~D\")'" outfile (- next 1) outfile next)))))
    (list (list "repl" "tmac.scm")
-	 (list "snd -noinit" "make-index.scm")
+	 (list "snd -noinit" "tpeak.scm")
 	 (list "repl" "tref.scm")
+	 (list "snd -noinit" "make-index.scm")
 	 (list "repl" "tauto.scm")
 	 (list "repl" "teq.scm")
 	 (list "repl" "s7test.scm")
-	 (list "repl" "tlet.scm")
 	 (list "repl" "lt.scm")
-	 (list "repl" "tform.scm")
 	 (list "repl" "tcopy.scm")
-	 (list "repl" "tmap.scm")
+	 (list "repl" "tread.scm")
+	 (list "repl" "tform.scm")
+	 (list "repl" "tlet.scm")
 	 (list "repl" "tfft.scm")
+	 (list "repl" "tmap.scm")
 	 (list "repl" "tsort.scm")
 	 (list "repl" "titer.scm")
 	 ;(list "repl" "lg.scm")
